--         FILE NAME: bootrom.vhdl
--       ENTITY NAME: boot_rom
-- ARCHITECTURE NAME: behave
--          REVISION: A
--
--       DESCRIPTION: 64 byte x 8 bit ROM to down a Monitor
--                    program on reset
--
--Written by John Kent for the mc6801 processor

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
-- use work.memory.all;

entity boot_rom is
  port (
    addr   : in   std_logic_vector(5 downto 0);
    data   : out  std_logic_vector(7 downto 0)
  );
end entity boot_rom;

architecture basic of boot_rom is
  constant width   : integer := 8;
  constant memsize : integer := 64;

  type rom_array is array(0 to memsize-1) of std_logic_vector(width-1 downto 0);

  constant rom_data : rom_array :=
  ( "11001110", "11111111", "11100111", -- FFC0 - CE FFE7  RESET LDX #MSG-1
    "11110110", "10000000", "00000100", -- FFC3 - F6 8004  POLL1 LDAB UARTCR
	 "01010111",                         -- FFC6 - 57             ASRB
	 "01010111",                         -- FFC7 - 57             ASRB
	 "00100100", "11111001",             -- FFC8 - 24 F9          BCC POLL1
	 "10100110", "00000001",             -- FFCA - A6 01          LDAA 1,X
	 "00100111", "00000110",             -- FFCC - 27 06          BEQ POLL2
	 "00001000",                         -- FFCE - 08             INX
	 "10110111", "10000000", "00000101", -- FFCF - B7 8005        STA UARTDR
    "00100110", "11101111",             -- FFD2 - 26 EF          BNE POLL1
	 "11110110", "10000000", "00000100", -- FFD4 - F6 8004  POLL2 LDAB UARTCR
	 "00000001",                         -- FFD7 - 01             NOP
	 "01010111",                         -- FFD8 - 57             ASRB
	 "00100100", "11111001",             -- FFD9 - 24 F9          BCC POLL2
	 "11001110", "10000000", "00000100", -- FFDB - CE 8004        LDX #$8004
	 "10100110", "00000001",             -- FFDE - A6 01          LDAA 1,X
	 "10100001", "00000011",             -- FFE0 - A1 03          CMPA 3,X
	 "00100111", "11011100",             -- FFE2 - 27 DC          BEQ RESET
	 "00100000", "11111110",             -- FFE4 - 20 FE          BRA *
	 "00000000", "01011000",             -- FFE6 - 00 58          fcb $00,'x'
    "01001000", "01100101", "01101100", -- FFE8 - 48 65 6c MSG   FCC "Hel"
	 "01101100", "01101111", "00100000", -- FFEB - 6c 6f 20       FCC "lo "
	 "01010111", "01101111", "01110010", -- FFEE - 57 6f 72       FCC "Wor"
    "01101100", "01100100",             -- FFF1 - 6c 64          FCC "ld"
    "00001010", "00001101", "00000000", -- FFF3 - 0a 0d 00       FCB LF,CR,NULL
    "00000000", "00000000",             -- FFF6 - 00 00          fcb null,null           
	 "11111111", "11000000",             -- FFF8 - FF C0          fdb $FFC0 ; Timer irq
	 "11111111", "11000000",             -- FFFA - FF C0          fdb $FFC0 ; Ext IRQ
	 "11111111", "11000000",             -- FFFC - FF C0          fcb $FFC0 ; SWI
	 "11111111", "11000000"              -- FFFE - FF C0          fdb $FFC0 ; Reset
);
begin
   data <= rom_data(conv_integer(addr)); 
end architecture basic;


